@@ -60,6 +60,8 @@ public:
 
     virtual bool getData(char** o_buffer, _int64* o_validBytes, _int64* o_startBytes = NULL);
 
+    virtual void dumpState();
+
     virtual void advance(_int64 bytes);
 
     virtual void nextBatch();
@@ -165,6 +167,16 @@ protected:
     const size_t         bufferSize;
 };
 
+void
+ReadBasedDataReader::dumpState()
+{
+    WriteErrorMessage("ReadBasedDataReader at 0x%llx state:\n", this);
+    WriteErrorMessage("\theaderBufferSize %lld, headerExtraSize %lld, amountAdvancedThroughUnderlyingStoreByUs %lld, nHeaderBuffersAllocated %d, hitEOFReadingHeader %d, bufferSize %lld\n",
+        headerBufferSize, headerExtraSize, amountAdvancedThroughUnderlyingStoreByUs, nHeaderBuffersAllocated, hitEOFReadingHeader, bufferSize);
+    WriteErrorMessage("\tnBuffers %d, headerBuffersOutstanding, %d, startedReadingHeader %d, extraBytes %lld, overflowBytes %lld, nextBatchID %d, nextBufferForReader %d, nextBufferForConsumer %d, lastBufferForConsumer %d\n",
+        nBuffers, headerBuffersOutstanding, startedReadingHeader, extraBytes, overflowBytes, nextBatchID, nextBufferForReader, nextBufferForConsumer, lastBufferForConsumer);
+}
+
 ReadBasedDataReader::ReadBasedDataReader(
     unsigned i_nBuffers,
     _int64 i_overflowBytes,
@@ -186,12 +198,20 @@ ReadBasedDataReader::ReadBasedDataReader(
     _ASSERT(extraFactor >= 0 && i_nBuffers > 0);
     bufferInfo = new BufferInfo[maxBuffers];
     extraBytes = max((_int64) 0, (_int64) ((bufferSize + overflowBytes) * extraFactor));
+
+    if (bufferSize <= 2 * (size_t)overflowBytes) {
+        WriteErrorMessage("ReadBasedDataReader::ReadBasedDataReader: the buffer size %lld isn't more than twice the overflow size %lld, so we will fail to make progres.  This is a code bug, please create a github issue.\n",
+            bufferSize, overflowBytes);
+        soft_exit(1);
+    }
+
     char* allocated = (char*) BigReserve(maxBuffers * (bufferSize + extraBytes + overflowBytes));
     BigCommit(allocated, nBuffers * (bufferSize + extraBytes + overflowBytes));
     if (NULL == allocated) {
         WriteErrorMessage("ReadBasedDataReader: unable to allocate IO buffer\n");
         soft_exit(1);
     }
+
     for (unsigned i = 0 ; i < nBuffers; i++) {
         bufferInfo[i].buffer = allocated;
         allocated += bufferSize + overflowBytes;
@@ -206,6 +226,7 @@ ReadBasedDataReader::ReadBasedDataReader(
         bufferInfo[i].holds = 0;
 		bufferInfo[i].headerBuffer = false;
     }
+
     nextBatchID = 1;
  
     nextBufferForConsumer = -1;
@@ -1331,6 +1352,8 @@ public:
         return fileName;
     }
 
+    virtual void dumpState();
+
 protected:
 
     // must hold the lock to call
@@ -1354,11 +1377,28 @@ protected:
 
 };
 
+void
+AsyncFileDataReader::dumpState()
+{
+    WriteErrorMessage("AsyncFileDataReader (0x%llx) state:\n", this);
+    WriteErrorMessage("\tfileName %s, fileSize %lld, readOffset %lld, endingOffset %lld\n",
+        fileName, fileSize, readOffset, endingOffset);
+    
+    ReadBasedDataReader::dumpState();
+}
+
 AsyncFileDataReader::AsyncFileDataReader(unsigned i_nBuffers, _int64 i_overflowBytes, double extraFactor, size_t bufferSpace) :
     ReadBasedDataReader(i_nBuffers, i_overflowBytes, extraFactor, bufferSpace), fileName(NULL), asyncFile(NULL), endingOffset(0)
 {
     readOffset = 0;
     bufferReaders = (AsyncFile::Reader**)malloc(sizeof(AsyncFile::Reader*) * maxBuffers);
+
+    if (NULL == bufferReaders) {
+        WriteErrorMessage("AsyncFileDataReader::AsyncFileDataReader(): malloc(%d) failed\n", sizeof(AsyncFile::Reader*) * maxBuffers);
+        soft_exit(1);
+        return; // Just to avoid the compiler warning
+    }
+
     for (unsigned i = 0; i < i_nBuffers; i++) {
         bufferReaders[i] = NULL;
     }
