@@ -42,23 +42,10 @@ ReadFASTAGenome(
 	const char* const*opt_out_alt_names,
 	int				 opt_out_alt_names_count,
 	GenomeDistance	 maxSizeForAutomaticALT,
-	bool             autoAlt);
-
-//
-// The FASTA appending functions return whether the write was successful.
-// 
-// WARNING: They write very long lines.
-// According to Wikipedia, a FASTA file's line limit should be 120, or better, 79.
-// Unix workaround if the piece names aren't too long: 'fold -w 79'.
-//
-
-    bool
-AppendFASTAGenome(const Genome *, FILE *fasta);
-
-//
-// This is arbitrary; is there some existing convention?
-//
-inline const char *diploidFASTASexPrefix(bool male)
-{
-    return male ? "PATERNAL|" : "MATERNAL|";
-}
+	bool             autoAlt,
+    char            **alt_liftover_contig_names,
+	unsigned		*alt_liftover_contig_flags,
+	char			**alt_liftover_proj_contig_names,
+	unsigned		*alt_liftover_proj_contig_offsets,
+	char			**alt_liftover_proj_cigar,
+	int				 alt_liftover_count);
